import glob
import os
import requests  # type: ignore
import json
from requests.auth import HTTPBasicAuth  # type: ignore
import argh  # type: ignore

def get_ssh_keys(username:str) -> tuple[bool, str, list[str]|None, list[str]|None]:

    with open("config.json", "r") as file:
        config = json.load(file)

    token_url = f"{config['keycloak_url']}/realms/master/protocol/openid-connect/token"
    token_data = {
        "grant_type": "password",
        "username": config["admin_username"],
        "password": config["admin_password"],
    }
    users_url = f"{config['keycloak_url']}/admin/realms/master/users"

    # Get token
    try:
        response = requests.post(
            token_url,
            data=token_data,
            auth=HTTPBasicAuth(config["client_id"], config["client_secret"]),
        )
        response.raise_for_status()

    except requests.exceptions.HTTPError:
        return False, "SSO connection broken. No token.", None, None

    access_token = response.json()["access_token"]
    headers = {
        "Authorization": f"Bearer {access_token}",
        "Content-Type": "application/json",
    }

    # Check if user exists
    params = {"username": username, "exact": "true"}

    try:
        response = requests.get(users_url, headers=headers, params=params)
        response.raise_for_status()

        # Response is a list of users matching the criteria
        users = response.json()

        if len(users) == 0:
            return False, f"User {username} does not exists.", None, None

        if len(users) > 1:
            return False, f"User {username} can not be identified.", None, None

    except requests.exceptions.HTTPError:
        return False, "Communication with SSO server failed.", None, None

    user_id = users[0]["id"]

    # Get user details
    user_url = f"{users_url}/{user_id}"
    try:
        response = requests.get(user_url, headers=headers)
        response.raise_for_status()
        user_data = response.json()
    except requests.exceptions.HTTPError as e:
        return False, f"Failed to get user details: {str(e)}", None, None

    if "attributes" not in user_data:
        user_data["attributes"] = dict()
    
    if "Overleaf_SSH_Public_Key" not in user_data["attributes"]:
        return True, "No keys in SSO", None, None

    if "Overleaf_SSH_Private_Key" not in user_data["attributes"]:
        return True, "No keys in SSO", None, None
        
    if user_data["attributes"]["Overleaf_SSH_Public_Key"] is None or user_data["attributes"]["Overleaf_SSH_Private_Key"] is None:
        return True, "No keys in SSO", None, None

    return True, "OK", user_data["attributes"]["Overleaf_SSH_Public_Key"], user_data["attributes"]["Overleaf_SSH_Private_Key"]



def main(username:str, path:str = "/downloads"):

    if len(username) == 0:
        exit(1)

    public_key_sso = None
    private_key_sso = None

    status, info_string, public_key_sso, private_key_sso = get_ssh_keys(username=username)

    if status is False:
        print(username)
        print(info_string)
        exit(1)

    if public_key_sso is None or private_key_sso is None:
        exit(1)

    if status:
        if len(public_key_sso) != 1:
            exit(1)
        if len(private_key_sso) != 1:
            exit(1)

        public_key_sso = public_key_sso[0].strip()
        private_key_sso = private_key_sso[0].strip()

        assert os.path.isdir(os.path.join(path, username, ".ssh"))
        assert os.path.isfile(os.path.join(path, username, ".ssh", "hajtex.pub")) is False
        assert os.path.isfile(os.path.join(path, username, ".ssh", "hajtex")) is False

        public_keys_name:str = os.path.join(path, username, ".ssh", "hajtex.pub")
        private_keys_name:str = os.path.join(path, username, ".ssh", "hajtex")
        with open(public_keys_name, 'w') as file:
            file.write(public_key_sso)
            if not public_key_sso.endswith('\n'):
                file.write('\n')

        with open(private_keys_name, 'w') as file:
            file.write(private_key_sso)
            if not private_key_sso.endswith('\n'):
                file.write('\n')
        exit(0)

    else:
        print(username)
        print(info_string)
    exit(1)


if __name__ == "__main__":
    argh.dispatch_command(main)

