import glob
import os
import requests  # type: ignore
import json
from requests.auth import HTTPBasicAuth  # type: ignore
import argh  # type: ignore

def set_ssh_keys(username:str, public_key:str, private_key:str, overwrite:bool=False) -> tuple[bool, str]:

    if public_key is None and private_key is None:
        return False, "No SSH keys provided to set."

    with open("config.json", "r") as file:
        config = json.load(file)

    token_url = f"{config['keycloak_url']}/realms/master/protocol/openid-connect/token"
    token_data = {
        "grant_type": "password",
        "username": config["admin_username"],
        "password": config["admin_password"],
    }
    users_url = f"{config['keycloak_url']}/admin/realms/master/users"

    # Get token
    try:
        response = requests.post(
            token_url,
            data=token_data,
            auth=HTTPBasicAuth(config["client_id"], config["client_secret"]),
        )
        response.raise_for_status()

    except requests.exceptions.HTTPError:
        return False, "SSO connection broken. No token."

    access_token = response.json()["access_token"]
    headers = {
        "Authorization": f"Bearer {access_token}",
        "Content-Type": "application/json",
    }

    # Check if user exists
    params = {"username": username, "exact": "true"}

    try:
        response = requests.get(users_url, headers=headers, params=params)
        response.raise_for_status()

        # Response is a list of users matching the criteria
        users = response.json()

        if len(users) == 0:
            return False, f"User {username} does not exists."

        if len(users) > 1:
            return False, f"User {username} can not be identified."

    except requests.exceptions.HTTPError:
        return False, "Communication with SSO server failed."

    user_id = users[0]["id"]

    # Get user details
    user_url = f"{users_url}/{user_id}"
    try:
        response = requests.get(user_url, headers=headers)
        response.raise_for_status()
        user_data = response.json()
    except requests.exceptions.HTTPError as e:
        return False, f"Failed to get user details: {str(e)}"

    if "attributes" not in user_data:
        user_data["attributes"] = dict()

    if "Overleaf_SSH_Public_Key" in user_data["attributes"] and user_data["attributes"]["Overleaf_SSH_Public_Key"] is not None:
        if len(user_data["attributes"]["Overleaf_SSH_Public_Key"]) > 0 and overwrite is False:
            print("== SKIPPED (OVERWRITE DISABLED) ==")
            return True, ""

    user_data["attributes"]["Overleaf_SSH_Public_Key"] = [public_key]
    user_data["attributes"]["Overleaf_SSH_Private_Key"] = [private_key]

    # Modify user attributes
    try:
        response = requests.put(user_url, headers=headers, json=user_data)
        response.raise_for_status()
    except requests.exceptions.HTTPError as e:
        return False, f"Failed to update user attributes: {str(e)}"

    return True, "OK"

def main(username:str, path:str = "/downloads"):

    if len(username) == 0:
        exit(1)

    print(username)

    public_keys_name:str = os.path.join(path, username, ".ssh", "hajtex.pub")
    private_keys_name:str = os.path.join(path, username, ".ssh", "hajtex")

    assert os.path.isdir(os.path.join(path, username, ".ssh"))
    assert os.path.isfile(public_keys_name)
    assert os.path.isfile(private_keys_name)

    with open(public_keys_name,'r') as file:
        public_key = file.read()

    with open(private_keys_name,'r') as file:
        private_key = file.read()

    status, info_string = set_ssh_keys(username=username, public_key=public_key, private_key=private_key)

    if status:
        exit(0)
    else:
        exit(1)

if __name__ == "__main__":
    argh.dispatch_command(main)
