import glob
import os
import requests  # type: ignore
import json
from requests.auth import HTTPBasicAuth  # type: ignore

def set_ssh_keys(username:str, public_key: str, private_key: str, overwrite: bool) -> tuple[bool, str, list[str]|None, list[str]|None]:

    if public_key is None and private_key is None:
        return False, "No SSH keys provided to set.", None, None

    with open("config.json", "r") as file:
        config = json.load(file)

    token_url = f"{config['keycloak_url']}/realms/master/protocol/openid-connect/token"
    token_data = {
        "grant_type": "password",
        "username": config["admin_username"],
        "password": config["admin_password"],
    }
    users_url = f"{config['keycloak_url']}/admin/realms/master/users"

    # Get token
    try:
        response = requests.post(
            token_url,
            data=token_data,
            auth=HTTPBasicAuth(config["client_id"], config["client_secret"]),
        )
        response.raise_for_status()

    except requests.exceptions.HTTPError:
        return False, "SSO connection broken. No token.", None, None

    access_token = response.json()["access_token"]
    headers = {
        "Authorization": f"Bearer {access_token}",
        "Content-Type": "application/json",
    }

    # Check if user exists
    params = {"username": username, "exact": "true"}

    try:
        response = requests.get(users_url, headers=headers, params=params)
        response.raise_for_status()

        # Response is a list of users matching the criteria
        users = response.json()

        if len(users) == 0:
            return False, f"User {username} does not exists.", None, None

        if len(users) > 1:
            return False, f"User {username} can not be identified.", None, None

    except requests.exceptions.HTTPError:
        return False, "Communication with SSO server failed.", None, None

    user_id = users[0]["id"]

    # Get user details
    user_url = f"{users_url}/{user_id}"
    try:
        response = requests.get(user_url, headers=headers)
        response.raise_for_status()
        user_data = response.json()
    except requests.exceptions.HTTPError as e:
        return False, f"Failed to get user details: {str(e)}", None, None

    if "attributes" not in user_data:
        user_data["attributes"] = dict()

    if "Overleaf_SSH_Public_Key" in user_data["attributes"] and user_data["attributes"]["Overleaf_SSH_Public_Key"] is not None:
        if len(user_data["attributes"]["Overleaf_SSH_Public_Key"]) > 0 and overwrite is False:
            print("== SKIPPED (OVERWRITE DISABLED) ==")
            return True, "", user_data["attributes"]["Overleaf_SSH_Public_Key"], user_data["attributes"]["Overleaf_SSH_Private_Key"]

    user_data["attributes"]["Overleaf_SSH_Public_Key"] = [public_key]
    user_data["attributes"]["Overleaf_SSH_Private_Key"] = [private_key]

    # Modify user attributes
    try:
        response = requests.put(user_url, headers=headers, json=user_data)
        response.raise_for_status()
    except requests.exceptions.HTTPError as e:
        return False, f"Failed to update user attributes: {str(e)}"

    return True, "OK", user_data["attributes"]["Overleaf_SSH_Public_Key"], user_data["attributes"]["Overleaf_SSH_Private_Key"]

path: str = "/docker/compose/scp_git_bridge/downloads/*"
overwrite: bool = False

path_list: list[str] = list(glob.glob(path))

for path_entry in path_list:
    username = path_entry.split("/")[-1]
    print(username)
    authorized_keys_name:str = os.path.join(path_entry, ".ssh", "authorized_keys")
    public_keys_name:str = os.path.join(path_entry, ".ssh", "hajtex.pub")
    private_keys_name:str = os.path.join(path_entry, ".ssh", "hajtex")

    assert os.path.isfile(authorized_keys_name)
    assert os.path.isfile(public_keys_name)
    assert os.path.isfile(private_keys_name)

    with open(public_keys_name,'r') as file:
        public_key = file.read()

    with open(private_keys_name,'r') as file:
        private_key = file.read()

    with open(authorized_keys_name,'r') as file:
        authorized_keys = file.read()

    status, info_string, public_key_sso, private_key_sso = set_ssh_keys(username=username, public_key=public_key, private_key=private_key, overwrite=overwrite)
    if public_key_sso is not None and private_key_sso is not None:

        assert len(public_key_sso) == 1
        assert len(private_key_sso) == 1

        public_key_sso = public_key_sso[0].strip()
        private_key_sso = private_key_sso[0].strip()

        if public_key_sso in authorized_keys:
            print("== authorized_keys is OKAY ==")
        else:
            # Append the public_key_sso to authorized_keys file
            with open(authorized_keys_name, 'w') as file:
                # Ensure proper formatting with newlines
                if authorized_keys and not authorized_keys.endswith('\n'):
                    authorized_keys += '\n'

                # Append the new key with a newline at the end
                file.write(authorized_keys + public_key_sso)

                if not public_key_sso.endswith('\n'):
                    file.write('\n')

        if public_key_sso in public_key:
            print("== public key is OKAY ==")
        else:
            with open(public_keys_name, 'w') as file:
                file.write(public_key_sso)

                if not public_key_sso.endswith('\n'):
                    file.write('\n')


        if private_key_sso in private_key:
            print("== private key is OKAY ==")
        else:
            with open(private_keys_name, 'w') as file:
                file.write(private_key_sso)

                if not private_key_sso.endswith('\n'):
                    file.write('\n')

        print("--------------------------------")
